/*
 * Class: CodeGeneration 
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.codegen;

import org.apache.bcel.generic.*;
import org.apache.bcel.Constants;
import org.apache.bcel.generic.CodeExceptionGen;

import edu.purdue.cs.bloat.cfg.Block;
import edu.purdue.cs.bloat.cfg.FlowGraph;
import edu.purdue.cs.bloat.cfg.Handler;
import edu.purdue.cs.bloat.cfg.ReplaceTarget;
import edu.purdue.cs.bloat.cfg.Subroutine;
import edu.purdue.cs.bloat.editor.MethodRef;
import edu.purdue.cs.bloat.optimize.MethodState;
import edu.purdue.cs.bloat.tree.AddressStoreStmt;
import edu.purdue.cs.bloat.tree.ArithExpr;
import edu.purdue.cs.bloat.tree.ArrayLengthExpr;
import edu.purdue.cs.bloat.tree.ArrayRefExpr;
import edu.purdue.cs.bloat.tree.CallMethodExpr;
import edu.purdue.cs.bloat.tree.CallStaticExpr;
import edu.purdue.cs.bloat.tree.CastExpr;
import edu.purdue.cs.bloat.tree.CatchExpr;
import edu.purdue.cs.bloat.tree.CheckExpr;
import edu.purdue.cs.bloat.tree.ConstantExpr;
import edu.purdue.cs.bloat.tree.DefExpr;
import edu.purdue.cs.bloat.tree.Expr;
import edu.purdue.cs.bloat.tree.ExprStmt;
import edu.purdue.cs.bloat.tree.FieldExpr;
import edu.purdue.cs.bloat.tree.GotoStmt;
import edu.purdue.cs.bloat.tree.IfCmpStmt;
import edu.purdue.cs.bloat.tree.IfStmt;
import edu.purdue.cs.bloat.tree.IfZeroStmt;
import edu.purdue.cs.bloat.tree.InitStmt;
import edu.purdue.cs.bloat.tree.InstanceOfExpr;
import edu.purdue.cs.bloat.tree.JsrStmt;
import edu.purdue.cs.bloat.tree.JumpStmt;
import edu.purdue.cs.bloat.tree.LabelStmt;
import edu.purdue.cs.bloat.tree.LocalExpr;
import edu.purdue.cs.bloat.tree.MemExpr;
import edu.purdue.cs.bloat.tree.MonitorStmt;
import edu.purdue.cs.bloat.tree.NegExpr;
import edu.purdue.cs.bloat.tree.NewArrayExpr;
import edu.purdue.cs.bloat.tree.NewExpr;
import edu.purdue.cs.bloat.tree.NewMultiArrayExpr;
import edu.purdue.cs.bloat.tree.Node;
import edu.purdue.cs.bloat.tree.PhiCatchStmt;
import edu.purdue.cs.bloat.tree.PhiJoinStmt;
import edu.purdue.cs.bloat.tree.PhiStmt;
import edu.purdue.cs.bloat.tree.ReplaceVisitor;
import edu.purdue.cs.bloat.tree.RetStmt;
import edu.purdue.cs.bloat.tree.ReturnAddressExpr;
import edu.purdue.cs.bloat.tree.ReturnExprStmt;
import edu.purdue.cs.bloat.tree.ReturnStmt;
import edu.purdue.cs.bloat.tree.ShiftExpr;
import edu.purdue.cs.bloat.tree.StackExpr;
import edu.purdue.cs.bloat.tree.StackManipStmt;
import edu.purdue.cs.bloat.tree.StackOptimizationData;
import edu.purdue.cs.bloat.tree.StaticFieldExpr;
import edu.purdue.cs.bloat.tree.Stmt;
import edu.purdue.cs.bloat.tree.StoreExpr;
import edu.purdue.cs.bloat.tree.SwitchStmt;
import edu.purdue.cs.bloat.tree.ThrowStmt;
import edu.purdue.cs.bloat.tree.Tree;
import edu.purdue.cs.bloat.tree.TreeVisitor;
import edu.purdue.cs.bloat.tree.UCExpr;
import edu.purdue.cs.bloat.tree.VarExpr;
import edu.purdue.cs.bloat.tree.ZeroCheckExpr;
import edu.purdue.cs.bloat.util.Assert;
import edu.purdue.cs.bloat.util.ImmutableIterator;

import java.util.*;

/**
 * CodeGenerator performs some final optimizations and is used (via a visitor)
 * to generate bytecode for the contents of a control flow graph.
 * 
 * @see FlowGraph
 */
public class CodeGenerator extends TreeVisitor {
    public static boolean DEBUG = false;

    public static boolean USE_PERSISTENT = false; // Generate _nowb

    /**
     * Use information about placement of local variables to eliminate loads and
     * stores in favor of stack manipulations
     */
    public static boolean OPT_STACK = false;

    public static boolean DB_OPT_STACK = false;

    // Statics from UseMapStore.
    public static boolean PRESERVE_DEBUG = false;

    public static boolean UNIQUE_HANDLERS = false;

    public static boolean OPT_STACK_2 = false; // byte-code level stack opt

    protected MethodGen method;

    InstructionFactory instFactory = null; // FACTORY

    protected MethodState state = null;

    protected Set visited;

    protected Map postponedInstructions;

    protected Block next;

    private InstructionHandle lastAdded = null; // remembers the location of the

    // last added insruction

    private Map handleMap = new LinkedHashMap();

    protected int stackHeight; // The current height of the stack

    StackOptimizationData currentSO; // object used to determine where to

    // apply
    // stack optimization

    /**
     * Constructor that takes the whole MethodState.
     * 
     * This allows the code generator to be invoked in some very nice ways.
     * 
     * @param state
     *            The method state for which bytecode is to be generated.
     */
    public CodeGenerator(MethodState state) {
        this(state.methodGen());
        this.state = state;
    }

    /**
     * Constructor.
     * 
     * @param method
     *            The method for which bytecode is generated.
     */
    public CodeGenerator(MethodGen method) {
        this.method = method;
        // method.setInstructionList(new InstructionList());
        // method.removeExceptionHandlers();
        // method.removeLocalVariables();
        // method.removeLineNumbers();

        this.postponedInstructions = new LinkedHashMap();

        instFactory = new InstructionFactory(method.getConstantPool()); // FACTORY
    }

    public CodeGenerator generateCode() {
        method.setInstructionList(new InstructionList());
        method.removeExceptionHandlers();
        method.removeLocalVariables();
        method.removeLineNumbers();
        generateCode(state.controlFlowGraph());
        return this;
    }

    public void generateCode(FlowGraph cfg) {
        cfg.visit(this);
    }

    // If Codegen worked by writing out the FlowGraph to bytecode in trace
    // order we would be left
    // with branch targets into a no longer correct InstructionList. These
    // could be translated to
    // the new InstructionHandles using a map of old to new Label
    // statements.
    // To avoid branch targets into a no longer existing InstructionList we
    // commit all Label
    // instructions first (in trace order) we also maintain a map of old to
    // new Label instructions.
    // In this way when we encounter a Label statement we moved
    // lastAdded(the insertion point for
    // committed instructions) to the appropriate InstructionHandle and can
    // avoid possible branch
    // targets into a invalid InstructionList --Arrin

    // This method allows us the fetch the new Label(InstructionHandle) from
    // the map.
    private InstructionHandle getLabelInstructionHandle(InstructionHandle ih) {
        InstructionHandle target = (InstructionHandle) handleMap.get(ih);
        Assert.isTrue(target != null,
                "Unable to find a corresponding InstructionHandle for block"
                        + ih);
        return target;
    }

    /**
     * Visits the nodes in the method's control flow graph and ensures that
     * information about the method's basic blocks is consistent and correct.
     * 
     * @param cfg
     *            The control flow graph associated with this method.
     */
    public void visitFlowGraph(final FlowGraph cfg) {
        // Generate the code.

        visited = new LinkedHashSet();
        visited.add(cfg.source());
        visited.add(cfg.sink());

        Iterator e = cfg.trace().iterator();

        Assert.isTrue(e.hasNext(), "trace is empty");

        stackHeight = 0; // At beginning of method stack has height 0
        Block block = null;
        while (e.hasNext()) {
            block = (Block) e.next();
            // Make sure the first label is marked as starting a block
            // and the rest are marked as not starting a block.
            block.visit(new TreeVisitor() {
                public void visitLabelStmt(LabelStmt stmt) {
                    FlowGraph.setLabel(stmt.label());
                    InstructionHandle ih;
                    ih = method.getInstructionList().append(new NOP());
                    // Unfortunately as we don't yet know what instruction
                    // to add it has to be a NOP
                    FlowGraph.setLabel(ih);
                    handleMap.put(stmt.label(), ih);
                    // map the old Label to the new one
                    // add NOP here and add the InstructionHandle to the Map
                }

                public void visitStmt(Stmt stmt) {
                }
            });
        }

        e = cfg.trace().iterator();

        block = (Block) e.next();

        // Visit each block in the method (via the trace in the method's CFG)
        // and ensure that the first (and ONLY the first) label in the code
        // is marked as starting a block.
        while (block != null) {
            if (e.hasNext()) {
                next = (Block) e.next();

            } else {
                next = null;
            }

            if (CodeGenerator.DEBUG) {
                System.out.println("code for " + block);
            }

            // Generate the code for each block
            visited.add(block);
            // currentSO is the StackOptimizationData object that discerns
            // where dups may be used instead of loads
            if (OPT_STACK)
                currentSO = block.stackOptimizer();
            // lastAdded = method.getInstructionList().getStart();
            block.visitChildren(this);

            block = next;
        }

        Assert.isTrue(visited.size() == cfg.size(),
                "did not visit all blocks while generating code");

        next = null;
        visited = null;

        // Go through the catch blocks and determine the what the
        // protected regions are that correspond to the catch blocks.
        // Create Handler objects to represent the protected regions.

        Iterator iter = cfg.catchBlocks().iterator();

        while (iter.hasNext()) {

            Block catchBlock = (Block) iter.next();
            Handler handler = (Handler) cfg.handlersMap().get(catchBlock);

            Type type = handler.catchType();

            if (type.equals(Type.NULL)) {
                type = null;
            }

            // First block in protected block
            Block begin = null;
            Block last = null;
            Iterator blocks = cfg.trace().iterator();

            while (blocks.hasNext()) {
                block = (Block) blocks.next();

                if (handler.protectedBlocks().contains(block)) {
                    if (begin == null) {
                        begin = block;
                    }
                    last = block;

                } else if (begin != null) {
                    // The block is no longer protected, its the end of the
                    // protected region
                    // Assigned the last handle of the previous block
                    InstructionHandle temp = getLabelInstructionHandle(last
                            .label());
                    if (temp.getNext() != null
                            && !FlowGraph.label(temp.getNext())) {
                        temp = temp.getNext();
                    }
                    InstructionHandle beginTarget = getLabelInstructionHandle(begin
                            .label());
                    InstructionHandle catchTarget = getLabelInstructionHandle(catchBlock
                            .label());
                    method.addExceptionHandler(beginTarget,// temp,
                            ((InstructionHandle) handleMap.get(catchBlock
                                    .label())).getPrev(), catchTarget,
                            (ObjectType) type);
                    begin = null;
                    last = null;
                }
            }
        }

        FlowGraph.removeNOPs(method);
        method.getInstructionList().setPositions();
        method.setMaxStack();
        method.setMaxLocals();
        method.update();

        if (DEBUG) {
            System.out.println("Method:");
            System.out.println(method);
            for (InstructionHandle ih = method.getInstructionList().getStart(); ih != null; ih = ih
                    .getNext()) {
                if (FlowGraph.label(ih))
                    System.out.print("(L)");
                else
                    System.out.print("   ");
                System.out.print(" " + ih.getPosition() + ": "
                        + ih.getInstruction());
                if (ih.getAttribute("Def") != null)
                    System.out.print("(" + ih.getAttribute("Def") + ")");
                System.out.println("");
            }
            // System.out.println(method.getMethod().getCode());
            System.out.println();
            System.out.println("LocalVariables: ");
            LocalVariableGen[] locals = method.getLocalVariables();
            for (int i = 0; i < locals.length; i++) {
                System.out.println("  " + locals[i]);
            }
            System.out.println("LocalVariable size: " + method.getMaxLocals());
            System.out.println("Stack size: " + method.getMaxStack());
            CodeExceptionGen[] excptns = method.getExceptionHandlers();
            if (excptns != null) {
                System.out.println("ExceptionHandlers: ");
                for (int j = 0; j < excptns.length; j++)
                    System.out.println("  " + excptns[j]);
            }
        }

    }

    public CodeGenerator simplifyControlFlow() {
        simplifyControlFlow(state.controlFlowGraph());
        return this;
    }

    /**
     * Simplifies the control flow of a method by changing jump and return
     * statements into gotos where appropriate.
     */
    public void simplifyControlFlow(final FlowGraph cfg) {
        // Remove any blocks from the CFG that consist of solely jumps
        removeEmptyBlocks(cfg);

        cfg.visit(new TreeVisitor() {
            public void visitJsrStmt(JsrStmt stmt) {
                final Subroutine sub = stmt.sub();

                // If there is only 1 path through the sub, replace both
                // the jsr and the ret with gotos.
                if (sub.numPaths() == 1) {
                    Block exit = sub.exit();

                    // Remember that it is not required for a subroutine to
                    // have
                    // a ret. So, no exit block may be identified and we'll
                    // have to make sure one exists.
                    if (exit != null) {
                        JumpStmt oldJump = (JumpStmt) exit.tree().lastStmt();
                        JumpStmt jump = new GotoStmt(stmt.follow());
                        jump.catchTargets().addAll(oldJump.catchTargets());
                        oldJump.replaceWith(jump);
                    }

                    JumpStmt jump = new GotoStmt(sub.entry());
                    jump.catchTargets().addAll(stmt.catchTargets());
                    stmt.replaceWith(jump);

                    // The subroutine is no longer really a subroutine
                    cfg.removeSub(sub);

                    // Clean up the CFG by removing all AddressStoreStmts
                    // that
                    // store the address of the "removed" subroutine.
                    cfg.visit(new TreeVisitor() {
                        Iterator iter;

                        public void visitTree(Tree tree) {
                            iter = tree.stmts().iterator();

                            while (iter.hasNext()) {
                                Stmt s = (Stmt) iter.next();

                                if (s instanceof AddressStoreStmt) {
                                    AddressStoreStmt store = (AddressStoreStmt) s;

                                    if (store.sub() == sub) {
                                        iter.remove();
                                    }
                                }
                            }
                        }
                    });
                }
            }

            public void visitStmt(Stmt stmt) {
            }
        });
    }

    public CodeGenerator replacePhis() {
        replacePhis(state.controlFlowGraph());
        return this;
    }

    /**
     * Replace PhiStmts with copies that accomplish what the PhiStmts represent.
     * Then remove the PhiStmts from the control flow graph.
     */
    public void replacePhis(final FlowGraph cfg) {
        replaceCatchPhis(cfg);
        replaceJoinPhis(cfg);

        // Remove the phis.
        cfg.visit(new TreeVisitor() {
            public void visitTree(Tree tree) {
                Iterator e = tree.stmts().iterator();

                while (e.hasNext()) {
                    Stmt s = (Stmt) e.next();

                    if (s instanceof PhiStmt) {
                        e.remove();
                    }
                }
            }
        });
    }

    /**
     * Replace each PhiCatchStmt with assignments at its operands' defs.
     */
    private void replaceCatchPhis(final FlowGraph cfg) {
        cfg.visit(new TreeVisitor() {
            LinkedHashMap seen = new LinkedHashMap();

            public void visitFlowGraph(FlowGraph graph) {
                Iterator iter = graph.catchBlocks().iterator();

                // Examine each block that begins an exception handler
                while (iter.hasNext()) {
                    Block block = (Block) iter.next();
                    block.visit(this);
                }
            }

            public void visitPhiCatchStmt(PhiCatchStmt phi) {
                LocalExpr target = (LocalExpr) phi.target();
                int index = target.index();

                Iterator iter = phi.operands().iterator();

                // Examine every operand of the PhiCatchStmt. If necessary,
                // insert copies of the operand to the target after the last
                // occurrence of the operand.
                while (iter.hasNext()) {
                    LocalExpr expr = (LocalExpr) iter.next();
                    LocalExpr def = (LocalExpr) expr.def();

                    if (def == null) {
                        continue;
                    }

                    if (DEBUG) {
                        System.out.println("inserting for " + phi + " at "
                                + def);
                    }

                    BitSet s = (BitSet) seen.get(def);

                    if (s == null) {
                        s = new BitSet();
                        seen.put(def, s);

                        final BitSet t = s;

                        // Visit the parent expression and make note of which
                        // local variables were encountered in StoreExprs. That
                        // is, have we already generated a copy for the operand
                        // of
                        // interest?
                        def.parent().visit(new TreeVisitor() {
                            public void visitStoreExpr(StoreExpr expr) {
                                if (DEBUG) {
                                    System.out.println("    merging with "
                                            + expr);
                                }

                                Expr lhs = expr.target();
                                Expr rhs = expr.expr();

                                if (lhs instanceof LocalExpr) {
                                    t.set(((LocalExpr) lhs).index());
                                }

                                if (rhs instanceof LocalExpr) {
                                    t.set(((LocalExpr) rhs).index());

                                } else if (rhs instanceof StoreExpr) {
                                    // Visit RHS. LHS be ignored by
                                    // visitNode.
                                    super.visitStoreExpr(expr);
                                }
                            }

                            public void visitNode(Node node) {
                            }
                        });
                    }

                    // If we've already inserted a copy (StoreStmt) for the
                    // local variable, skip it
                    if (s.get(index)) {
                        continue;
                    }

                    s.set(index);

                    Assert.isTrue(def != null);

                    if (def.parent() instanceof Stmt) {
                        // Insert a new Stmt to copy into the target

                        Stmt stmt = (Stmt) def.parent();
                        Stmt store = createStore(target, def);
                        def.block().tree().addStmtAfter(store, stmt);

                    } else {
                        Assert.isTrue(def.parent() instanceof StoreExpr);

                        // Replace s := r with s := (t := r)
                        StoreExpr p = (StoreExpr) def.parent();
                        Expr rhs = p.expr();

                        if (rhs instanceof LocalExpr
                                && ((LocalExpr) rhs).index() == def.index()) {
                            // No need to insert a copy. Just change the
                            // index
                            // (local variable to which LocalExpr is
                            // assigned) to be
                            // the same as the target
                            def.setIndex(index);

                        } else {
                            rhs.setParent(null);

                            // Copy the rhs into the target
                            StoreExpr store = new StoreExpr((LocalExpr) target
                                    .clone(), rhs, rhs.type());

                            p.visit(new ReplaceVisitor(rhs, store));
                        }
                    }
                }
            }

            public void visitStmt(Stmt stmt) {
            }
        });
    }

    /**
     * Replace PhiJoinStmts with assignments at the end of the predecessor
     * blocks. Note that from now on the FUD chains are broken since there can
     * be more than one def of a variable.
     */
    private void replaceJoinPhis(final FlowGraph cfg) {
        // Go in trace order since liveness was computed under this
        // assumption.

        Iterator iter = cfg.trace().iterator();

        while (iter.hasNext()) {
            Block block = (Block) iter.next();

            if (block == cfg.sink()) {
                continue;
            }

            block.visit(new TreeVisitor() {
                public void visitPhiJoinStmt(PhiJoinStmt stmt) {
                    // If an operand of the Phi statement is undefined,
                    // insert
                    // code to assign 0 to the operand. The value should
                    // never
                    // be used, but the verifier will squawk about using an
                    // undefined local variable.

                    Iterator preds = cfg.preds(stmt.block()).iterator();

                    while (preds.hasNext()) {
                        Block pred = (Block) preds.next();

                        Expr operand = stmt.operandAt(pred);

                        if (stmt.target() instanceof LocalExpr
                                && operand instanceof LocalExpr) {

                            LocalExpr t = (LocalExpr) stmt.target();
                            LocalExpr s = (LocalExpr) operand;

                            if (t.index() == s.index()) {
                                // The target and the operand are already
                                // allocated to
                                // the same variable. Don't bother making a
                                // copy.
                                continue;
                            }
                        }

                        final Tree tree = pred.tree();

                        // Insert stores before the last stmt to ensure
                        // we don't redefine locals used the the branch stmt.
                        final Stmt last = tree.lastStmt();

                        last.visitChildren(new TreeVisitor() {
                            // The last statement in the block should be a
                            // jump. If
                            // the jump statement contains an expression,
                            // replace
                            // that expression with a stack variable. Before
                            // the
                            // jump, insert a store of the expression into
                            // the stack
                            // variable. This is done so that the store to
                            // the
                            // PhiJoinStmt's operand does not interfere with
                            // any
                            // local variables that might appear in the
                            // expression.
                            //
                            // operand = ...
                            // JUMP (exp)
                            // |
                            // v
                            // target = PhiJoin(operand)
                            // ...
                            // Becomes
                            //
                            // operand = ...
                            // var = exp
                            // target = operand
                            // JUMP (var)
                            // |
                            // v
                            // target = PhiJoin(operand) // Removed later
                            // ...

                            public void visitExpr(Expr expr) {
                                StackExpr var = tree.newStack(expr.type());
                                var.setValueNumber(expr.valueNumber());

                                Node p = expr.parent();
                                expr.setParent(null);
                                p.visit(new ReplaceVisitor(expr, var));

                                var = (StackExpr) var.clone();
                                StoreExpr store = new StoreExpr(var, expr, expr
                                        .type());
                                store.setValueNumber(expr.valueNumber());

                                Stmt storeStmt = new ExprStmt(store);
                                storeStmt.setValueNumber(expr.valueNumber());

                                tree.addStmtBeforeJump(storeStmt);
                            }

                            public void visitStackExpr(StackExpr expr) {
                            }
                        });

                        Stmt store = createStore(stmt.target(), operand);

                        if (CodeGenerator.DEBUG) {
                            System.out.println("CodeGen: insert for " + stmt
                                    + " " + store + " in " + pred);
                        }

                        tree.addStmtBeforeJump(store);
                    }
                }

                public void visitStmt(Stmt stmt) {
                }
            });
        }
    }

    /**
     * Removes blocks that contain no other statements than gotos, jumps,
     * returns, or labels. Other blocks that are invovled with the blocks being
     * removed are updated appropriately.
     */
    private void removeEmptyBlocks(final FlowGraph cfg) {

        Set emptyBlocks = new LinkedHashSet();

        Iterator e = cfg.nodes().iterator();

        BLOCKS: while (e.hasNext()) {
            Block block = (Block) e.next();

            // Collect any blocks that contain only gotos,
            // jsrs, rets, or labels.
            Iterator stmts = block.tree().stmts().iterator();

            while (stmts.hasNext()) {
                Stmt stmt = (Stmt) stmts.next();

                if (stmt instanceof GotoStmt || stmt instanceof JsrStmt
                        || stmt instanceof RetStmt || stmt instanceof LabelStmt) {
                    continue;
                }

                // The block contains something other than the above, it is
                // not empty.
                continue BLOCKS;
            }

            emptyBlocks.add(block);
        }

        // We want to keep the source, init, and sink blocks even if
        // they're empty
        emptyBlocks.remove(cfg.source());
        emptyBlocks.remove(cfg.init());
        emptyBlocks.remove(cfg.sink());

        // Did the CFG change?
        boolean changed = true;

        while (changed) {
            changed = false;

            // Exclude the blocks that are control dependent on other
            // blocks.
            Set empty = new LinkedHashSet(emptyBlocks);
            empty.removeAll(cfg.iteratedPdomFrontier(cfg.nodes()));

            e = empty.iterator();

            EMPTY: while (e.hasNext()) {
                Block block = (Block) e.next();

                Stmt last = block.tree().lastStmt();

                Assert.isTrue(last instanceof GotoStmt
                        || last instanceof JsrStmt || last instanceof RetStmt);

                if (last instanceof GotoStmt) {
                    // All a block does is jump to another block
                    //
                    // jmp ... L
                    // L: goto M
                    // =>
                    // jmp ... M
                    Block target = ((GotoStmt) last).target();

                    Iterator preds = new ImmutableIterator(cfg.preds(block));

                    while (preds.hasNext()) {
                        Block pred = (Block) preds.next();
                        Assert.isTrue(pred != cfg.source());

                        Stmt predLast = pred.tree().lastStmt();
                        predLast.visit(new ReplaceTarget(block, target));

                        cfg.removeEdge(pred, block);
                        cfg.addEdge(pred, target);

                        changed = true;
                    }

                } else if (last instanceof RetStmt) {
                    // All a subroutine does is return

                    Iterator preds = new ImmutableIterator(cfg.preds(block));

                    while (preds.hasNext()) {
                        Block pred = (Block) preds.next();
                        Assert.isTrue(pred != cfg.source());

                        Stmt predLast = pred.tree().lastStmt();

                        if (predLast instanceof JsrStmt) {
                            // The previous block is the jsr...
                            //
                            // jsr L ret to M
                            // M: ...
                            // L: ret // The body of the subroutine is empty
                            // =>
                            // goto M
                            // M: ...

                            JsrStmt stmt = (JsrStmt) predLast;

                            JumpStmt jump = new GotoStmt(stmt.follow());
                            jump.catchTargets().addAll(stmt.catchTargets());
                            stmt.replaceWith(jump);

                            stmt.sub().removePathsContaining(pred);

                        } else if (predLast instanceof GotoStmt) {
                            // The previous block ends in a goto. Move the
                            // ret up
                            // into the previous block, update catch targets
                            // of any
                            // exceptions thrown by the block terminated by
                            // the
                            // jump, and update the subroutine's exit block
                            // to be
                            // the previous block (in which the ret now
                            // resides).

                            JumpStmt jump = (RetStmt) last.clone();
                            jump.catchTargets().addAll(
                                    ((JumpStmt) predLast).catchTargets());
                            predLast.replaceWith(jump);
                            ((RetStmt) last).sub().setExit(pred);
                        }

                        // Remove the block from the CFG
                        cfg.succs(pred).remove(block);
                        cfg.succs(pred).addAll(cfg.succs(block));

                        changed = true;
                    }

                } else if (last instanceof JsrStmt) {
                    // All the block does is a jsr
                    //
                    // goto L
                    // L: jsr M
                    // =>
                    // jsr M
                    // L: jsr M
                    Iterator preds = new ImmutableIterator(cfg.preds(block));

                    while (preds.hasNext()) {
                        Block pred = (Block) preds.next();
                        Assert.isTrue(pred != cfg.source());

                        Stmt predLast = pred.tree().lastStmt();

                        if (predLast instanceof GotoStmt) {
                            JsrStmt stmt = (JsrStmt) last;

                            JumpStmt jump = new JsrStmt(stmt.sub(), stmt
                                    .follow());
                            jump.catchTargets().addAll(
                                    ((JumpStmt) predLast).catchTargets());
                            predLast.replaceWith(jump);

                            // The block is no longer a viable caller of the
                            // subroutine
                            stmt.sub().removePathsContaining(block);
                            stmt.sub().addPath(pred, stmt.follow());

                            cfg.addEdge(pred, stmt.sub().entry());
                            cfg.removeEdge(pred, block);

                            changed = true;
                        }
                    }
                } else if (last == null) {// MOD There is only a LabelSmt
                    // in block
                    Iterator preds = new ImmutableIterator(cfg.preds(block));

                    while (preds.hasNext()) {
                        Block pred = (Block) preds.next();
                        Assert.isTrue(pred != cfg.source());
                        cfg.succs(pred).remove(block);
                    }
                    changed = true;
                } else {
                    throw new RuntimeException();
                }
            }

            if (changed) {
                cfg.removeUnreachable();

                // Remove any empty blocks that we've already deleted.
                emptyBlocks.retainAll(cfg.nodes());
            }
        }
    }

    public CodeGenerator allocateReturnAddresses(RegisterAllocator alloc) {
        Assert.isNotNull(alloc);
        allocReturnAddresses(state.controlFlowGraph(), alloc);
        return this;
    }

    /**
     * Allocate "registers" (LocalVariables) for the return addresses for each
     * subroutine in the method.
     * 
     * @param cfg
     *            Control flow graph for the method
     * @param alloc
     *            Allocation (and information about) the local variables in the
     *            method.
     * 
     * @see LocalVariable
     * @see LocalExpr
     */
    public void allocReturnAddresses(FlowGraph cfg,
            final RegisterAllocator alloc) {
        // Allocate registers for the returnAddresses. Don't bother trying
        // to minimize the number of locals, just get a new one.
        Iterator e = cfg.subroutines().iterator();

        while (e.hasNext()) {
            Subroutine sub = (Subroutine) e.next();
            LocalVariableGen var = alloc.newLocal(sub.returnAddress());
            sub.setLocalVariable(var);
        }
    }

    /**
     * Create a ExprStmt that initializes a target variable to a default value
     * based on the type of the target.
     */
    protected Stmt createUndefinedStore(VarExpr target) {
        if (target.type() instanceof ReferenceType) {
            return new ExprStmt(new StoreExpr(target, new ConstantExpr(null,
                    Type.OBJECT), target.type()));
        }

        if (Assert.isIntegral(target.type())) {
            return new ExprStmt(new StoreExpr(target, new ConstantExpr(
                    new Integer(0), Type.INT), target.type()));
        }

        if (target.type().equals(Type.LONG)) {
            return new ExprStmt(new StoreExpr(target, new ConstantExpr(
                    new Long(0), Type.LONG), target.type()));
        }

        if (target.type().equals(Type.FLOAT)) {
            return new ExprStmt(new StoreExpr(target, new ConstantExpr(
                    new Float(0.0F), Type.FLOAT), target.type()));
        }

        if (target.type().equals(Type.DOUBLE)) {
            return new ExprStmt(new StoreExpr(target, new ConstantExpr(
                    new Double(0.0), Type.DOUBLE), target.type()));
        }
        throw new RuntimeException("Illegal type: " + target.type());
    }

    /**
     * Returns an ExprStmt that contains a store of the source into the target.
     */
    protected Stmt createStore(VarExpr target, Expr source) {
        target = (VarExpr) target.clone();
        // WHY clone them?

        // Source is an undefined variable, initialize it
        // if (source instanceof VarExpr && source.def() == null){
        // return createUndefinedStore(target);
        // }
        return new ExprStmt(new StoreExpr(target, (Expr) source.clone(), // Why
                // clone
                // them?
                target.type()));
    }

    /*
     * Using an InstructionVisitor generate the code...
     */

    // Several of the visit methods contain code for stack
    // optimization (place dups and swaps and eliminate temporary
    // variables). Nodes where swaps are to be placed are so
    // marked. The markings may appear at IfCmpStmt, InitStmt,
    // StoreExpr, ArithExpr, ArrayRefExpr, CallMethodExpr,
    // CallStaticExpr, NewMultiArrayExpr, ShiftExpr.
    public void visitExpr(Expr expr) {
        throw new RuntimeException("Unhandled expression type: "
                + expr.getClass().getName());
    }

    public void visitExprStmt(ExprStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        if (!(stmt.expr() instanceof StoreExpr)) {
            if (!stmt.expr().type().equals(Type.VOID)) {
                if (stmt.expr().type().getSize() == 2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new POP2());
                    stackHeight -= 2;

                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new POP());
                    stackHeight -= 1;
                }
            }
        }
    }

    public void visitInitStmt(InitStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }
    }

    public void visitGotoStmt(GotoStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        // genPostponed(stmt);

        Block targetBlock = stmt.target();
        if (targetBlock != next) {
            InstructionHandle target = getLabelInstructionHandle(targetBlock
                    .label());
            lastAdded = method.getInstructionList().append(lastAdded,
                    new GOTO(target));
        }
    }

    public void visitIfCmpStmt(IfCmpStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        Block t = stmt.trueTarget();
        Block f = stmt.falseTarget();

        if (f == next) {
            // Fall through to the false branch.
            genIfCmpStmt(stmt);

        } else if (t == next) {
            // Fall through to the true branch.
            stmt.negate();
            genIfCmpStmt(stmt);

        } else {
            // Generate a goto to the false branch after the if statement.
            genIfCmpStmt(stmt);

            // method.addLabel(method.newLabelTrue()); //Tom changed to say
            // "True"
            InstructionHandle falseTarget = getLabelInstructionHandle(f.label());
            lastAdded = method.getInstructionList().append(lastAdded,
                    new GOTO(falseTarget));
            FlowGraph.setLabel(lastAdded);// Just in case it's needed
        }
    }

    private void genIfCmpStmt(IfCmpStmt stmt) {
        BranchInstruction newInst;
        stmt.visitChildren(this);

        // genPostponed(stmt);

        int cmp = stmt.comparison();
        // Find the appropriate LabelStmt in the new list.
        InstructionHandle trueTarget = getLabelInstructionHandle(stmt
                .trueTarget().label());

        if (stmt.left().type() instanceof ReferenceType) {
            Assert.isTrue((stmt.right().type() instanceof ReferenceType),
                    "Illegal statement: " + stmt);

            switch (cmp) {
            case IfStmt.EQ:
                newInst = new IF_ACMPEQ(trueTarget);
                break;
            case IfStmt.NE:
                newInst = new IF_ACMPNE(trueTarget);
                break;
            default:
                throw new RuntimeException();
            }

        } else {
            Assert.isTrue(Assert.isIntegral(stmt.left().type()),
                    "Illegal statement: " + stmt);
            Assert.isTrue(Assert.isIntegral(stmt.right().type()),
                    "Illegal statement: " + stmt);

            switch (cmp) {
            case IfStmt.EQ:
                newInst = new IF_ICMPEQ(trueTarget);
                break;
            case IfStmt.NE:
                newInst = new IF_ICMPNE(trueTarget);
                break;
            case IfStmt.GT:
                newInst = new IF_ICMPGT(trueTarget);
                break;
            case IfStmt.GE:
                newInst = new IF_ICMPGE(trueTarget);
                break;
            case IfStmt.LT:
                newInst = new IF_ICMPLT(trueTarget);
                break;
            case IfStmt.LE:
                newInst = new IF_ICMPLE(trueTarget);
                break;
            default:
                throw new RuntimeException();
            }
        }

        lastAdded = method.getInstructionList().append(lastAdded, newInst);
        stackHeight -= 2;
    }

    public void visitIfZeroStmt(IfZeroStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        Block t = stmt.trueTarget();
        Block f = stmt.falseTarget();

        if (f == next) {
            // Fall through to the false branch.
            genIfZeroStmt(stmt);
        } else if (t == next) {
            // Fall through to the true branch.
            stmt.negate();
            genIfZeroStmt(stmt);
        } else {
            // Generate a goto to the false branch after the if statement.
            genIfZeroStmt(stmt);
            InstructionHandle falseTarget = getLabelInstructionHandle(f.label());

            lastAdded = method.getInstructionList().append(lastAdded,
                    new GOTO(falseTarget));
            FlowGraph.setLabel(lastAdded);// Just in case it's needed
        }
    }

    private void genIfZeroStmt(IfZeroStmt stmt) {
        BranchInstruction inst;
        InstructionHandle trueTarget = getLabelInstructionHandle(stmt
                .trueTarget().label());

        stmt.expr().visit(this);

        // genPostponed(stmt);

        int cmp = stmt.comparison();

        if (stmt.expr().type() instanceof ReferenceType) {
            switch (cmp) {
            case IfStmt.EQ:
                inst = new IFNULL(trueTarget);
                break;
            case IfStmt.NE:
                inst = new IFNONNULL(trueTarget);
                break;
            default:
                throw new RuntimeException();
            }

        } else {
            Assert.isTrue(Assert.isIntegral(stmt.expr().type()),
                    "Illegal statement: " + stmt);

            switch (cmp) {
            case IfStmt.EQ:
                inst = new IFEQ(trueTarget);
                break;
            case IfStmt.NE:
                inst = new IFNE(trueTarget);
                break;
            case IfStmt.GT:
                inst = new IFGT(trueTarget);
                break;
            case IfStmt.GE:
                inst = new IFGE(trueTarget);
                break;
            case IfStmt.LT:
                inst = new IFLT(trueTarget);
                break;
            case IfStmt.LE:
                inst = new IFLE(trueTarget);
                break;
            default:
                throw new RuntimeException();
            }
        }
        lastAdded = method.getInstructionList().append(lastAdded, inst);
        stackHeight -= 1;
    }

    public void visitLabelStmt(LabelStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        lastAdded = getLabelInstructionHandle(stmt.label());
        // change the lastAdded InstructionHandle so that instructions are now
        // added
        // after this label inside the block this label starts. --Arrin
    }

    public void visitMonitorStmt(MonitorStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        if (stmt.kind() == MonitorStmt.ENTER) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new MONITORENTER());
            stackHeight -= 1;

        } else if (stmt.kind() == MonitorStmt.EXIT) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new MONITOREXIT());
            stackHeight -= 1;

        } else {
            throw new IllegalArgumentException();
        }
    }

    public void visitPhiStmt(PhiStmt stmt) {
        throw new RuntimeException("Cannot generate code for " + stmt);
    }

    // I haven't done anything with the persistentcy code in BLOAT thiss
    // code has just been
    // commented out and left untouched. --Arrin

    /*
     * public void visitRCExpr(RCExpr expr) { expr.visitChildren(this);
     * 
     * genPostponed(expr); // Move the rc forward as far as possible. // // For
     * example, for the expression: // // rc(x).m(rc(a).b) // // we want to
     * generate: // // aload x // aload a // rc 0 // getfield <A.b> // rc 1 //
     * invoke <X.m> // // rather than: // // aload x // rc 0 // aload a // rc 0 //
     * getfield <A.b> // invoke <X.m> // Instruction postpone = null;
     * 
     * Node parent = expr.parent(); // If the parent is wrapped in a
     * ZeroCheckExpr, then look at the // parent's parent.
     * 
     * if(parent instanceof ZeroCheckExpr) parent = parent.parent(); // If the
     * depth is going to be > 0, postpone the rc instruction // to just before
     * the getfield, putfield, invoke, xaload, xastore, etc. // If the stack
     * depth for the rc is going to be 0, the rc will (be) // the next
     * instruction generated anyway, so don't postpone.
     * 
     * if (parent instanceof ArrayRefExpr) { ArrayRefExpr p = (ArrayRefExpr)
     * parent;
     * 
     * if (expr == p.array()) { if (p.isDef()) { // a[i] := r // Stack at the
     * xastore: ... a i r postpone = new Instruction(opcx_rc, new
     * Integer(p.type().stackHeight() + 1)); } else { // use a[i] // Stack at
     * the xaload: ... a i postpone = new Instruction(opcx_rc, new Integer(1)); } } }
     * else if (parent instanceof CallMethodExpr) { CallMethodExpr p =
     * (CallMethodExpr) parent;
     * 
     * if (expr == p.receiver()) { // a.m(b, c) // Stack at the invoke: ... a b
     * c MemberRef method = p.method(); int depth =
     * method.nameAndType().type().stackHeight(); postpone = new
     * Instruction(opcx_rc, new Integer(depth)); } } else if (parent instanceof
     * FieldExpr) { FieldExpr p = (FieldExpr) parent;
     * 
     * if (expr == p.object()) { if (p.isDef()) { // a.b := r // Stack at the
     * putfield: ... a r postpone = new Instruction(opcx_rc, new
     * Integer(p.type().stackHeight())); } } }
     * 
     * if (postpone == null) { int depth = 0;
     * 
     * if(expr.expr() instanceof StackExpr) { // If the rc works on a StackExpr,
     * calculate its depth in the // stack. In all other cases, the rc will
     * operate on whatever // is on top of the stack. StackExpr stackVar =
     * (StackExpr) expr.expr(); depth = stackHeight - stackVar.index() - 1; }
     * 
     * method.addInstruction(opcx_rc, new Integer(depth)); } else {
     * postponedInstructions.put(parent, postpone); } }
     */

    /*
     * public void visitUCExpr(UCExpr expr) { expr.visitChildren(this);
     * 
     * if(true) return;
     * 
     * genPostponed(expr); // Move the uc forward as far as possible.
     * Instruction postpone = null;
     * 
     * Node parent = expr.parent(); // If the depth is going to be > 0, postpone
     * the uc instruction // to just before the putfield. If the stack depth for
     * the // uc is going to be 0, the uc will the next instruction // generated
     * anyway, so don't postpone.
     * 
     * if (parent instanceof FieldExpr) { FieldExpr p = (FieldExpr) parent;
     * 
     * if (expr == p.object()) { if (p.isDef()) { // a.b := r // Stack at the
     * putfield: ... a r if (expr.kind() == UCExpr.POINTER) { postpone = new
     * Instruction(opcx_aupdate, new Integer(p.type().stackHeight())); } else if
     * (expr.kind() == UCExpr.SCALAR) { postpone = new Instruction(opcx_supdate,
     * new Integer(p.type().stackHeight())); } else { throw new
     * RuntimeException(); } } } }
     * 
     * if (postpone == null) { int depth = 0;
     * 
     * if(expr.expr() instanceof StackExpr) { // If the UCExpr operates on a
     * stack variable, use that to // determine the depth. In all other cases,
     * use 0. StackExpr stackVar = (StackExpr) expr.expr(); depth = stackHeight -
     * stackVar.index() - 1; }
     * 
     * if (expr.kind() == UCExpr.POINTER) { method.addInstruction(opcx_aupdate,
     * new Integer(depth)); } else if (expr.kind() == UCExpr.SCALAR) {
     * method.addInstruction(opcx_supdate, new Integer(depth)); } else { throw
     * new RuntimeException(); } } else { postponedInstructions.put(parent,
     * postpone); } }
     */
    public void visitRetStmt(RetStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        // genPostponed(stmt);

        Subroutine sub = stmt.sub();
        Assert.isTrue(sub.returnAddress() != null);
        lastAdded = method.getInstructionList().append(lastAdded,
                new RET(sub.localVariable().getIndex()));
    }

    public void visitReturnExprStmt(ReturnExprStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        Type type = stmt.expr().type();

        // Stack should be empty after return
        lastAdded = method.getInstructionList().append(lastAdded,
                InstructionFactory.createReturn(type));
        stackHeight = 0;
    }

    public void visitReturnStmt(ReturnStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        // genPostponed(stmt);

        stmt.visitChildren(this);
        lastAdded = method.getInstructionList().append(lastAdded, new RETURN());

        // Stack height is zero after return
        stackHeight = 0;
    }

    public void visitStoreExpr(StoreExpr expr) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + expr);
        }

        MemExpr lhs = expr.target();
        Expr rhs = expr.expr();

        boolean returnsValue = !(expr.parent() instanceof ExprStmt);

        // eliminate the store if the both sides have the same target

        if (!returnsValue) {
            if (lhs instanceof LocalExpr && rhs instanceof LocalExpr) {

                // The second condition checks that the right sides have indeed
                // been stored-- otherwise (ie, if we're just keeping it on the
                // stack), we should not eliminate this store

                if (((LocalExpr) lhs).index() == ((LocalExpr) rhs).index()

                        && (!OPT_STACK || currentSO
                                .shouldStore(((LocalExpr) ((LocalExpr) rhs)
                                        .def())))) {
                    return;
                }
            }

            // Special case to handle L := L + k.
            // Generate "iinc L, k" instead of "iload L; ldc k; iadd; istore
            // L".
            //
            // TODO: for L := M + k, generate "iload M; istore L; iinc L,
            // k".
            //

            /*
             * This next special case was modified for stack optimization. If L
             * is marked for a dup, the fact that its value is never on the
             * stack means we don't have anything to dup-- we need to load
             * instead. (Things get more complicated if it was marked for a
             * dup_x2, but that's not likely to happen)
             */

            if (lhs instanceof LocalExpr && Assert.isIntegral(lhs.type())) {
                Integer value = null;
                // eliminate the store if the both sides have the same target

                int index = ((LocalExpr) lhs).index();

                if (rhs instanceof ArithExpr) {
                    ArithExpr arith = (ArithExpr) rhs;

                    Expr left = arith.left();
                    Expr right = arith.right();

                    if (left instanceof LocalExpr
                            && index == ((LocalExpr) left).index()
                            && right instanceof ConstantExpr) {

                        ConstantExpr c = (ConstantExpr) right;

                        if (c.value() instanceof Integer) {
                            value = (Integer) c.value();
                        }

                    } else if (right instanceof LocalExpr
                            && index == ((LocalExpr) right).index()
                            && left instanceof ConstantExpr
                            && arith.operation() == ArithExpr.ADD) {

                        // This will not work for x = c - x because it is not
                        // the
                        // same as x = x - c.

                        ConstantExpr c = (ConstantExpr) left;

                        if (c.value() instanceof Integer) {
                            value = (Integer) c.value();
                        }
                    }

                    if (value != null && arith.operation() == ArithExpr.SUB) {
                        value = new Integer(-value.intValue());
                    } else if (arith.operation() != ArithExpr.ADD) {
                        value = null;
                    }
                }

                if (value != null) {
                    int incr = value.intValue();

                    if (incr == 0) {
                        // No need to increment by 0.

                        // for a better understanding of what's going on in
                        // these
                        // additions, see VisitLocalExpr, where we do basically
                        // the
                        // same thing.

                        if (OPT_STACK) {
                            int dups, dup_x1s, dup_x2s;
                            dups = currentSO.dups((LocalExpr) lhs);
                            dup_x1s = currentSO.dup_x1s((LocalExpr) lhs);
                            dup_x2s = currentSO.dup_x2s((LocalExpr) lhs);
                            for (int i = 0; i < dup_x2s; i++) {
                                // This is really awful, but be consoled in that
                                // it is
                                // highly improbable to happen... this is just
                                // to make correct code in the chance that we
                                // have something like this.
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new LDC(0));
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new DUP_X2());
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new POP());
                                stackHeight += 1;
                            }
                            for (int i = 0; i < dup_x1s; i++) {
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new LDC(0));
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new SWAP());
                                stackHeight += 1;
                            }
                            for (int i = 0; i < dups; i++) {
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new LDC(0));
                                stackHeight += 1;
                            }
                        }
                        return;

                    } else if ((short) incr == incr) {
                        // Only generate an iinc if the increment fits in
                        // a short.
                        // method.addLocalVariable("codeGenLocal" + index,
                        // Type.INT, index, null, null);

                        lastAdded = method.getInstructionList().append(
                                lastAdded, new IINC(index, incr));

                        if (OPT_STACK) {
                            int dups, dup_x1s, dup_x2s;
                            dups = currentSO.dups((LocalExpr) lhs);
                            dup_x1s = currentSO.dup_x1s((LocalExpr) lhs);
                            dup_x2s = currentSO.dup_x2s((LocalExpr) lhs);
                            for (int i = 0; i < dup_x2s; i++) {
                                // method.addLocalVariable("codeGenLocal"
                                // + ((LocalExpr) lhs).index(), Type.INT,
                                // ((LocalExpr) lhs).index(), null, null);
                                lastAdded = method.getInstructionList().append(
                                        lastAdded,
                                        new ISTORE(((LocalExpr) lhs).index()));
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new DUP_X2());
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new POP());
                                stackHeight += 1;
                            }
                            for (int i = 0; i < dup_x1s; i++) {
                                // method.addLocalVariable("codeGenLocal"
                                // + ((LocalExpr) lhs).index(), Type.INT,
                                // ((LocalExpr) lhs).index(), null, null);
                                lastAdded = method.getInstructionList().append(
                                        lastAdded,
                                        new ILOAD(((LocalExpr) lhs).index()));
                                lastAdded = method.getInstructionList().append(
                                        lastAdded, new SWAP());
                                stackHeight += 1;
                            }
                            for (int i = 0; i < dups; i++) {
                                lastAdded = method.getInstructionList().append(
                                        lastAdded,
                                        new ILOAD(((LocalExpr) lhs).index()));
                                // method.addLocalVariable("codeGenLocal"
                                // + ((LocalExpr) lhs).index(), Type.INT,
                                // ((LocalExpr) lhs).index(), null, null);

                                stackHeight += 1;
                            }
                        }
                        return;
                    }
                }
            }
        }

        // Generate, and return the value.
        lhs.visitChildren(this);
        rhs.visit(this);

        if (returnsValue) {
            if (lhs instanceof ArrayRefExpr) {
                // array index rhs --> rhs array index rhs
                if (rhs.type().getSize() == 2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2_X2());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP_X2());
                    stackHeight += 1;
                }

            } else if (lhs instanceof FieldExpr) {
                // object rhs --> rhs object rhs
                if (rhs.type().getSize() == 2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2_X1());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP_X1());
                    stackHeight += 1;
                }

            } else {
                // rhs --> rhs rhs
                if (rhs.type().getSize() == 2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP());
                    stackHeight += 1;
                }
            }
        }

        // genPostponed(expr);
        lhs.visitOnly(this);
    }

    public void visitAddressStoreStmt(AddressStoreStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        // genPostponed(stmt);

        Subroutine sub = stmt.sub();
        Assert.isTrue(sub.localVariable() != null);
        method.getInstructionList().append(
                new ASTORE(sub.localVariable().getIndex()));
        stackHeight -= 1;
    }

    public void visitJsrStmt(JsrStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }
        // genPostponed(stmt);

        Block entry = stmt.sub().entry();
        InstructionHandle entryIH = getLabelInstructionHandle(entry.label());

        lastAdded = method.getInstructionList().append(lastAdded,
                new JSR(entryIH));
        stackHeight += 1;

        if (stmt.follow() != next) {
            InstructionHandle follow = getLabelInstructionHandle(stmt.follow()
                    .label());
            lastAdded = method.getInstructionList().append(lastAdded,
                    new GOTO(follow));
        }
    }

    public void visitSwitchStmt(SwitchStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        InstructionHandle[] targets = new InstructionHandle[stmt.targets().length];
        InstructionHandle defaultTarget = getLabelInstructionHandle(stmt
                .defaultTarget().label());
        for (int i = 0; i < targets.length; i++) {
            targets[i] = getLabelInstructionHandle(stmt.targets()[i].label());
        }

        lastAdded = method.getInstructionList().append(lastAdded,
                new SWITCH(stmt.values(), targets, defaultTarget));
        stackHeight -= 1;
    }

    public void visitStackManipStmt(StackManipStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        // genPostponed(stmt);

        // All the children are stack variables, so don't so anything.

        switch (stmt.kind()) {
        case StackManipStmt.SWAP: {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new SWAP());
            break;
        }
        case StackManipStmt.DUP:
            lastAdded = method.getInstructionList()
                    .append(lastAdded, new DUP());
            stackHeight += 1;
            break;
        case StackManipStmt.DUP_X1:
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DUP_X1());
            stackHeight += 1;
            break;
        case StackManipStmt.DUP_X2:
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DUP_X2());
            stackHeight += 1;
            break;
        case StackManipStmt.DUP2:
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DUP2());
            stackHeight += 2;
            break;
        case StackManipStmt.DUP2_X1:
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DUP2_X1());
            stackHeight += 2;
            break;
        case StackManipStmt.DUP2_X2:
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DUP2_X2());
            stackHeight += 2;
            break;
        }
    }

    public void visitThrowStmt(ThrowStmt stmt) {
        if (CodeGenerator.DEBUG) {
            System.out.println("code for " + stmt);
        }

        stmt.visitChildren(this);

        // genPostponed(stmt);

        lastAdded = method.getInstructionList().append(lastAdded, new ATHROW());
    }

    /*
     * public void visitSCStmt(SCStmt stmt) { stmt.visitChildren(this);
     * genPostponed(stmt); method.addInstruction(opcx_aswizzle); stackHeight -=
     * 2; }
     *//*
         * public void visitSRStmt(SRStmt stmt) { stmt.visitChildren(this);
         * genPostponed(stmt); method.addInstruction(opcx_aswrange); stackHeight -=
         * 3; }
         */
    public void visitArithExpr(ArithExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        int[][] stackChange = new int[][] { { -1, -2, -1, -2 },
                { -1, -2, 0, 0 }, { -1, -2, -1, -2 }, { -1, -2, -1, -2 },
                { -1, -2, 0, 0 }, { -1, -2, -1, -2 }, { -1, -2, -1, -2 },
                { -1, -2, 0, 0 }, { 0, -3, 0, 0 }, { 0, 0, -1, -3 },
                { 0, 0, -1, -3 } };

        int type;

        if (Assert.isIntegral(expr.left().type())) {
            type = 0;

        } else if (expr.left().type().equals(Type.LONG)) {
            type = 1;

        } else if (expr.left().type().equals(Type.FLOAT)) {
            type = 2;

        } else if (expr.left().type().equals(Type.DOUBLE)) {
            type = 3;

        } else {
            throw new IllegalArgumentException("Can't generate code for type: "
                    + expr.left().type() + " (expr " + expr + ")");
        }

        switch (expr.operation()) {
        case ArithExpr.ADD:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IADD());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LADD());
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FADD());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DADD());
                break;
            }
            stackHeight += stackChange[0][type];
            break;
        case ArithExpr.AND:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IAND());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LAND());
                break;
            case 2:
            case 3:
                break;
            }
            stackHeight += stackChange[1][type];
            break;
        case ArithExpr.DIV:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IDIV());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LDIV());
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FDIV());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DDIV());
                break;
            }
            stackHeight += stackChange[2][type];
            break;
        case ArithExpr.MUL:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IMUL());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LMUL());
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FMUL());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DMUL());
                break;
            }
            stackHeight += stackChange[3][type];
            break;
        case ArithExpr.IOR:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IOR());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LOR());
                break;
            case 2:
            case 3:
                break;
            }
            stackHeight += stackChange[4][type];
            break;
        case ArithExpr.REM:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IREM());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LREM());
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FREM());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DREM());
                break;
            }
            stackHeight += stackChange[5][type];
            break;
        case ArithExpr.SUB:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new ISUB());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LSUB());
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FSUB());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DSUB());
                break;
            }
            stackHeight += stackChange[6][type];
            break;
        case ArithExpr.XOR:
            switch (type) {
            case 0:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IXOR());
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LXOR());
                break;
            case 2:
            case 3:
                break;
            }
            stackHeight += stackChange[7][type];
            break;
        case ArithExpr.CMP:
            switch (type) {
            case 0:
                break;
            case 1:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LCMP());
                break;
            case 2:
            case 3:
                break;
            }
            stackHeight += stackChange[8][type];
            break;
        case ArithExpr.CMPL:
            switch (type) {
            case 0:
            case 1:
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FCMPL());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DCMPL());
                break;
            }
            stackHeight += stackChange[9][type];
            break;
        case ArithExpr.CMPG:
            switch (type) {
            case 0:
            case 1:
                break;
            case 2:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new FCMPG());
                break;
            case 3:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new DCMPG());
                break;
            }
            stackHeight += stackChange[10][type];
            break;
        }
    }

    public void visitArrayLengthExpr(ArrayLengthExpr expr) {
        expr.visitChildren(this);
        lastAdded = method.getInstructionList().append(lastAdded,
                new ARRAYLENGTH());
    }

    public void visitArrayRefExpr(ArrayRefExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);

        Instruction inst;

        if (expr.isDef()) {

            inst = InstructionFactory.createArrayStore(expr.elementType());
            stackHeight = stackHeight - 2 - expr.elementType().getSize();
        } else {
            inst = InstructionFactory.createArrayLoad(expr.elementType());
            stackHeight = stackHeight - 2 + expr.elementType().getSize();
        }

        lastAdded = method.getInstructionList().append(lastAdded, inst);
    }

    public void visitCallMethodExpr(CallMethodExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);

        Instruction inst;
        String declaring = "";
        MethodRef meth = (MethodRef) expr.method();
        if (meth.declaringClass() instanceof ArrayType) {
            declaring = meth.declaringClass().getSignature().replace('/', '.');
        } else {
            declaring = ((ObjectType) meth.declaringClass()).getClassName();
        }

        String methName = meth.name();
        String signature = Type.getMethodSignature(meth.returnType(), meth
                .paramTypes());
        if (expr.kind() == CallMethodExpr.VIRTUAL) {
            int index = method.getConstantPool().lookupMethodref(declaring,
                    methName, signature);
            inst = new INVOKEVIRTUAL(index);
        } else if (expr.kind() == CallMethodExpr.NONVIRTUAL) {
            int index = method.getConstantPool().lookupMethodref(declaring,
                    methName, signature);
            inst = new INVOKESPECIAL(index);
        } else if (expr.kind() == CallMethodExpr.INTERFACE) {
            int index = method.getConstantPool().lookupInterfaceMethodref(
                    declaring, methName, signature);
            inst = new INVOKEINTERFACE(index, meth.paramTypes().length + 1);
            // +1 because it must include the 'this' argument as well
        } else {
            throw new IllegalArgumentException();
        }

        lastAdded = method.getInstructionList().append(lastAdded, inst);

        // Pop reciever object off stack
        stackHeight -= 1;

        // Pop each parameter off stack
        Expr[] params = expr.params();
        for (int i = 0; i < params.length; i++) {
            stackHeight -= params[i].type().getSize();
        }
    }

    public void visitCallStaticExpr(CallStaticExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        MethodRef meth = (MethodRef) expr.method();
        String declaring = meth.declaringClass().toString();
        String methName = meth.name();
        String signature = Type.getMethodSignature(meth.returnType(), meth
                .paramTypes());
        int index = method.getConstantPool().lookupMethodref(declaring,
                methName, signature);
        lastAdded = method.getInstructionList().append(lastAdded,
                new INVOKESTATIC(index));

        // Pop each parameter off stack
        Expr[] params = expr.params();
        for (int i = 0; i < params.length; i++) {
            stackHeight -= params[i].type().getSize();
        }
    }

    public void visitCastExpr(CastExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        if (expr.castType() instanceof ReferenceType) {
            ConstantPoolGen cpg = method.getConstantPool();
            if (expr.castType() instanceof ArrayType) {
                lastAdded = method.getInstructionList().append(
                        lastAdded,
                        new CHECKCAST(cpg.addArrayClass((ArrayType) expr
                                .castType())));
                return;
            } else {
                lastAdded = method.getInstructionList().append(
                        lastAdded,
                        new CHECKCAST(cpg
                                .addClass((ObjectType) expr.castType())));
                return;
            }
        }
        int opType = expr.expr().type().getType();
        int castType = expr.castType().getType();

        switch (opType) {
        case Constants.T_BYTE:
        case Constants.T_SHORT:
        case Constants.T_CHAR:
        case Constants.T_INT:
            switch (castType) {
            case Constants.T_BYTE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2B());
                return;
            case Constants.T_SHORT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2S());
                return;
            case Constants.T_CHAR:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2C());
                return;
            case Constants.T_INT:
                return;
            case Constants.T_LONG:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2L());
                stackHeight += 1;
                return;
            case Constants.T_FLOAT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2F());
                return;
            case Constants.T_DOUBLE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2D());
                stackHeight += 1;
                return;
            }
            throw new IllegalArgumentException("Can't generate cast for type "
                    + expr.castType());
        // new Type(castType));

        case Constants.T_LONG:
            switch (castType) {
            case Constants.T_BYTE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2B());
                return;
            case Constants.T_SHORT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2S());
                return;
            case Constants.T_CHAR:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2C());
                return;
            case Constants.T_INT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2I());
                stackHeight -= 1;
                return;
            case Constants.T_LONG:
                return;
            case Constants.T_FLOAT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2F());
                stackHeight -= 1;
                return;
            case Constants.T_DOUBLE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new L2D());
                return;
            }

            throw new IllegalArgumentException("Can't generate cast for type "
                    + expr.castType());
        // new Type(castType));

        case Constants.T_FLOAT:
            switch (castType) {
            case Constants.T_BYTE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2I());
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2B());
                return;
            case Constants.T_SHORT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2I());
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2S());
                return;
            case Constants.T_CHAR:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2I());
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2C());
                return;
            case Constants.T_INT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2I());
                return;
            case Constants.T_LONG:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2L());
                stackHeight += 1;
                return;
            case Constants.T_FLOAT:
                return;
            case Constants.T_DOUBLE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new F2D());
                stackHeight += 1;
                return;
            }

            throw new IllegalArgumentException("Can't generate cast for type "
                    + expr.castType());
        // new Type(castType));

        case Constants.T_DOUBLE:
            switch (castType) {
            case Constants.T_BYTE:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2B());
                return;
            case Constants.T_SHORT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2S());
                return;
            case Constants.T_CHAR:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2I());
                stackHeight -= 1;
                lastAdded = method.getInstructionList().append(lastAdded,
                        new I2C());
                return;
            case Constants.T_INT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2I());
                stackHeight -= 1;
                return;
            case Constants.T_LONG:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2L());
                return;
            case Constants.T_FLOAT:
                lastAdded = method.getInstructionList().append(lastAdded,
                        new D2F());
                return;
            case Constants.T_DOUBLE:
                return;
            }

            throw new IllegalArgumentException("Can't generate cast for type "
                    + expr.castType());
        // new Type(castType));
        default:
            throw new IllegalArgumentException("Can't generate cast from type "
                    + expr.expr().type());
        // new Type(castType));
        }
    }

    public void visitConstantExpr(ConstantExpr expr) {
        int index = -1;
        ConstantPoolGen cpg = method.getConstantPool();
        expr.visitChildren(this);
        Object value = expr.value();
        // genPostponed(expr);
        Type type = expr.type();

        if (type.equals(Type.NULL) || type.equals(Type.OBJECT) || value == null) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new ACONST_NULL());
        } else {
            lastAdded = method.getInstructionList().append(lastAdded,
                    instFactory.createConstant(value));
        }
        if (CodeGenerator.DEBUG)
            System.out.println("Adding to ConstantPool " + expr.type()
                    + " index: " + index + " size: " + cpg.getSize());
        stackHeight += expr.type().getSize();
    }

    public boolean nowb = false;

    public void visitFieldExpr(FieldExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);

        if (expr.isDef()) {

            boolean UC = false; // Do we need an UC?

            // Look at the FieldExpr's object for a UCExpr
            Node check = expr.object();
            while (check instanceof CheckExpr) {
                if (check instanceof UCExpr) {
                    UC = true;
                    break;
                }

                CheckExpr c = (CheckExpr) check;
                check = c.expr();
            }

            // Do we need to perform the write barrier?
            if (!UC && USE_PERSISTENT) {
                /*
                 * System.out.println("Emitting a putfield_nowb in " +
                 * this.method.declaringClass().classInfo().name() + "." +
                 * this.method.name());
                 */
                nowb = true;

                // I commented out the next line because it generated a compiler
                // error, and I figured it was just about some unimportant
                // persistance stuff --Tom
                // method.addInstruction(opcx_putfield_nowb, expr.field());

            } else {
                int index = -1;
                if (expr.field().declaringClass() instanceof ArrayType) {
                    index = method.getConstantPool().addFieldref(
                            expr.field().declaringClass().getSignature(),
                            expr.field().name(),
                            expr.field().type().getSignature());
                } else {
                    index = method.getConstantPool().addFieldref(
                            ((ObjectType) expr.field().declaringClass())
                                    .getClassName(), expr.field().name(),
                            expr.field().type().getSignature());
                }
                lastAdded = method.getInstructionList().append(lastAdded,
                        new PUTFIELD(index));
            }

            stackHeight -= 1; // object
            stackHeight -= expr.type().getSize();

        } else {
            int index = -1;
            if (expr.field().declaringClass() instanceof ArrayType) {
                index = method.getConstantPool()
                        .addFieldref(
                                expr.field().declaringClass().getSignature(),
                                expr.field().name(),
                                expr.field().type().getSignature());
            } else {
                index = method.getConstantPool().addFieldref(
                        ((ObjectType) expr.field().declaringClass())
                                .getClassName(), expr.field().name(),
                        expr.field().type().getSignature());
            }
            lastAdded = method.getInstructionList().append(lastAdded,
                    new GETFIELD(index));
            stackHeight -= 1; // pop object
            stackHeight += expr.type().getSize();
        }
    }

    public void visitInstanceOfExpr(InstanceOfExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
        ReferenceType checkType = (ReferenceType) expr.checkType();
        int index = -1;
        if (checkType instanceof ArrayType) {
            index = method.getConstantPool().addArrayClass(
                    (ArrayType) checkType);
        } else {
            index = method.getConstantPool().addClass((ObjectType) checkType);
        }
        lastAdded = method.getInstructionList().append(lastAdded,
                new INSTANCEOF(index));
    }

    public void visitLocalExpr(LocalExpr expr) {
        // genPostponed(expr);

        boolean cat2 = (expr.type().getSize() == 2); // how many stack
        // positions does this take up?

        Instruction inst = null;
        // assigned
        // a real value

        if (OPT_STACK && DB_OPT_STACK) {
            currentSO.infoDisplay(expr);
        }
        if (expr.isDef()) {

            if (!OPT_STACK || currentSO.shouldStore(expr)) {
                inst = InstructionFactory
                        .createStore(expr.type(), expr.index());
                stackHeight = stackHeight - expr.type().getSize();
            }
        } else {
            if (OPT_STACK && currentSO.onStack(expr)) { // don't load if it's
                // already on
                // the stack
                if (currentSO.shouldSwap(expr)) {
                    if (cat2)
                        throw new IllegalArgumentException(
                                "Can't swap for wide expression "
                                        + expr.toString() + " of type "
                                        + expr.type().toString());
                    else {
                        inst = new SWAP();
                        stackHeight -= 1;
                    }
                }
            } else {
                inst = InstructionFactory.createLoad(expr.type(), expr.index());
                stackHeight = stackHeight + expr.type().getSize();
            }
        }

        if (inst instanceof SWAP)
            lastAdded = method.getInstructionList().append(lastAdded, inst);
        // don't give a swap an operand
        else if (inst != null && !(expr.isDef())) {
            // if this is a load, we want
            // the load before any dups.
            lastAdded = method.getInstructionList().append(lastAdded, inst);
            // method.addLocalVariable("codeGenLocal" + expr.index(),
            // expr.type(),
            // expr.index(), null, null);

            if (CodeGenerator.OPT_STACK_2)// remember the definition of
                // this use
                rememberDef(lastAdded, expr);
        }

        if (OPT_STACK) {
            // generate dups for this value on top of the stack
            int dups, dup_x1s, dup_x2s;
            dups = currentSO.dups(expr);
            dup_x1s = currentSO.dup_x1s(expr);
            dup_x2s = currentSO.dup_x2s(expr);

            for (int i = 0; i < dup_x2s; i++)
                if (cat2) { // (cat2 is for wide types)
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2_X2());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP_X2());
                    stackHeight += 1;
                }
            for (int i = 0; i < dup_x1s; i++)
                if (cat2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2_X1());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP_X1());
                    stackHeight += 1;
                }
            for (int i = 0; i < dups; i++)
                if (cat2) {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP2());
                    stackHeight += 2;
                } else {
                    lastAdded = method.getInstructionList().append(lastAdded,
                            new DUP());
                    stackHeight += 1;
                }
        }

        // if we have an opcode for a def (i.e., a store), generate it
        if (inst != null && expr.isDef()) {
            // method.addLocalVariable("codeGenLocal" + expr.index(),
            // expr.type(),
            // expr.index(), null, null);

            lastAdded = method.getInstructionList().append(lastAdded, inst);

            if (CodeGenerator.OPT_STACK_2)
                rememberDef(lastAdded, expr);
        }

        if (OPT_STACK // if we shouldn't store,
                && !currentSO.shouldStore(expr)) { // an extra thing will
            // be
            if (cat2) { // on the stack. pop it
                lastAdded = method.getInstructionList().append(lastAdded,
                        new POP2());
                stackHeight -= 2;
            } else {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new POP());
                stackHeight -= 1;
            }
        }
        // (if this leaves a useless dup/pop combination, let peephole fix it)
        // method.addInstruction(opcode, new LocalVariable(expr.index()));
    }

    public void visitNegExpr(NegExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);

        if (Assert.isIntegral(expr.type())) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new INEG());
        } else if (expr.type().equals(Type.FLOAT)) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new FNEG());
        } else if (expr.type().equals(Type.LONG)) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new LNEG());
        } else if (expr.type().equals(Type.DOUBLE)) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new DNEG());
        } else {
            throw new IllegalArgumentException("Can't generate code for type: "
                    + expr.type() + " (expr " + expr + ")");
        }
    }

    public void visitNewArrayExpr(NewArrayExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        if (expr.elementType() instanceof ReferenceType) {
            int index = 0;
            if (expr.elementType() instanceof ObjectType)
                index = method.getConstantPool().addClass(
                        (ObjectType) expr.elementType());
            else
                index = method.getConstantPool().addArrayClass(
                        (ArrayType) expr.elementType());

            lastAdded = method.getInstructionList().append(lastAdded,
                    new ANEWARRAY(index));
        } else {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new NEWARRAY((BasicType) expr.elementType()));
        }
    }

    public void visitNewExpr(NewExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        int index = method.getConstantPool().addClass(
                (ObjectType) expr.objectType());
        lastAdded = method.getInstructionList().append(lastAdded,
                new NEW(index));
        stackHeight += 1;
    }

    public void visitNewMultiArrayExpr(NewMultiArrayExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);
        int index = 0;
        if (expr.elementType() instanceof ObjectType)
            index = method.getConstantPool().addClass(
                    (ObjectType) expr.elementType());
        else
            index = method.getConstantPool().addArrayClass(
                    (ArrayType) expr.elementType());

        lastAdded = method.getInstructionList().append(lastAdded,
                new MULTIANEWARRAY(index, (short) expr.dimensions().length));
        stackHeight -= expr.dimensions().length;
        stackHeight += 1;
    }

    public void visitReturnAddressExpr(ReturnAddressExpr expr) {
        // genPostponed(expr);
    }

    public void visitShiftExpr(ShiftExpr expr) {
        expr.visitChildren(this);

        // genPostponed(expr);

        if (Assert.isIntegral(expr.type())) {
            if (expr.dir() == ShiftExpr.LEFT) {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new ISHL());
                stackHeight -= 1;
            } else if (expr.dir() == ShiftExpr.RIGHT) {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new ISHR());
                stackHeight -= 1;
            } else {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new IUSHR());
                stackHeight -= 1;
            }
        } else if (expr.type().equals(Type.LONG)) {
            if (expr.dir() == ShiftExpr.LEFT) {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LSHL());
                stackHeight -= 1;
            } else if (expr.dir() == ShiftExpr.RIGHT) {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LSHR());
                stackHeight -= 1;
            } else {
                lastAdded = method.getInstructionList().append(lastAdded,
                        new LUSHR());
                stackHeight -= 1;
            }
        } else {
            throw new IllegalArgumentException("Can't generate code for type: "
                    + expr.type() + " (expr " + expr + ")");
        }
    }

    public void visitDefExpr(DefExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
    }

    public void visitCatchExpr(CatchExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
    }

    public void visitStackExpr(StackExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
    }

    public void visitStaticFieldExpr(StaticFieldExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
        int index = method.getConstantPool().addFieldref(
                ((ObjectType) expr.field().declaringClass()).getClassName(),
                expr.field().name(), expr.field().type().getSignature());

        if (expr.isDef()) {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new PUTSTATIC(index));
            stackHeight -= expr.type().getSize();
        } else {
            lastAdded = method.getInstructionList().append(lastAdded,
                    new GETSTATIC(index));
            stackHeight += expr.type().getSize();
        }
    }

    public void visitZeroCheckExpr(ZeroCheckExpr expr) {
        expr.visitChildren(this);
        // genPostponed(expr);
    }

    private void rememberDef(InstructionHandle ih, LocalExpr expr) {
        Node def = expr.def();
        if (def != null)
            ih.addAttribute("Def", def);
    }
}
