/*
 * Class: InstructionVisitor 
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.editor;

import org.apache.bcel.generic.Instruction;

/**
 * The visitor pattern allows functionality to be added to a number of classes
 * (or in this case one class, <tt>Instruction</tt>, that can vary in
 * behavior) without modifying the classes themselves. Additionally, the visitor
 * pattern simulates double dispatching. For instance <tt>visit</tt> method of
 * <tt>Instruction</tt> calls a particular method of
 * <tt>InstructionVisitor</tt> based on the <tt>Instruction</tt>'s opcode.
 * <p>
 * <tt>InstructionVisitor</tt> provides an interface for performing actions
 * based on the instruction type. Classes implementing this interface should not
 * be able to miss any of the instruction types. This interface was created as
 * an alternative to having 138 different subtypes of Instruction.
 * 
 * @see Instruction#visit
 * @see edu.purdue.cs.bloat.tree.Tree
 * 
 * @author Nate Nystrom (<a
 *         href="mailto:nystrom@cs.purdue.edu">nystrom@cs.purdue.edu</a>)
 */
public interface InstructionVisitor {
    public void visitInst(Instruction inst);
    // public void visit_rc(Instruction inst);
    // public void visit_aupdate(Instruction inst);
    // public void visit_supdate(Instruction inst);
    // public void visit_aswizzle(Instruction inst);
    // public void visit_aswrange(Instruction inst);
}
