/*
 * Class: MethodRef 
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.editor;

import org.apache.bcel.generic.*;

/**
 * MemberRef represents a method or field (as a <tt>NameAndType</tt>) and the
 * class (as a <tt>Type</tt>) in which it is declared.
 * 
 * @author Arrin Daley (<a
 *         href="mailto:arrin@cs.anu.edu.au">arrin@cs.anu.edu.au</a>)
 * 
 * I found it rather confusing having MemberRef refer to both fields and methods
 * so I have subclassed MemberRef with MethodRef for use when refering to
 * methods. --Arrin
 * @see MemberRef
 */

/*
 * BCEL types only refer to a field or object type but BLOAT types can refer to
 * both a method and a field as a result I made this class to store Method
 * reference information
 */

public class MethodRef extends MemberRef {

    private final Type[] params;

    private final Type returnType;

    public MethodRef(ReferenceType declaringClass, NameAndType nameAndType,
            Type returnType, Type[] paramTypes) {
        super(declaringClass, nameAndType);
        this.params = paramTypes;
        this.returnType = returnType;
    }

    public Type[] paramTypes() {
        return params;
    }

    public Type returnType() {
        return returnType;
    }

    public Type type() {
        return returnType;
    }

    public boolean equals(Object obj) {
        if (super.equals(obj) && obj instanceof MethodRef) {
            MethodRef methRef = (MethodRef) obj;
            Type[] paramTest = methRef.paramTypes();
            if (returnType.equals(methRef.returnType())
                    && params.length == paramTest.length) {
                for (int i = 0; i < params.length; i++) {
                    if (!params[i].equals(paramTest[i]))
                        return false;
                }
            } else {
                return false;
            }
        } else
            return false;

        return true;
    }

    public String toString() {
        // Take advantage of PRINT_TRUNCATED in Type
        String className = declaringClass().toString();
        return "<" + "Method" + " " + className + "." + name() + " "
                + Type.getMethodSignature(returnType, params) + ">";
    }

    public String getSignature() {
        return Type.getMethodSignature(returnType, params);
    }
}
