/*
 * Class: CodeGen   
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.optimize;

import java.text.DateFormat;
import java.util.Date;
import edu.purdue.cs.bloat.codegen.RegisterAllocator;

import edu.purdue.cs.bloat.codegen.CodeGenerator;

/**
 * CodeGen generates the code after the other optimizations have been performed.
 * It's an Optimization itself, so you have to make sure it gets put in the list
 * of optimizations to perform.
 * <p>
 * 
 * CodeGen will print debugging information if the system property
 * <tt>edu.purdue.cs.bloat.DEBUG</tt> is set to <tt>true</tt>.
 * 
 * @author Chris Bennetts
 * 
 */
public class CodeGen implements Optimization {
    private RegisterAllocator alloc = null;

    private static DateFormat dateFormat = DateFormat.getDateTimeInstance(
            DateFormat.SHORT, DateFormat.LONG);

    private final boolean DEBUG;

    public CodeGen(RegisterAllocator alloc) {
        DEBUG = "true".equals(System.getProperty("edu.purdue.cs.bloat.DEBUG"));
        this.alloc = alloc;
    }

    public void transform(MethodState state) {
        // Liveness and RegisterAllocation take place in CodeGenerator
        CodeGenerator codegen = new CodeGenerator(state);

        // Register allocation must occur before replacePhis().
        printTraceMessage("  Register allocation");

        codegen.replacePhis();
        printPostPhisMessage(state);

        codegen.simplifyControlFlow();

        codegen.allocateReturnAddresses(alloc);
        printPostAllocationMessage(state);
        codegen.generateCode();
    }

    public String traceMessage(String dateString) {

        return "  Code Generation: " + dateString;
    }

    public String preDebugMessage() {
        return null;
    }

    public String postDebugMessage() {
        return null;
    }

    private void printTraceMessage(String root) {
        if (true)
            return;

        String dateString = dateFormat.format(new Date());
        System.out.println(root + ": " + dateString);
    }

    private void printPostPhisMessage(MethodState state) {
        if (DEBUG) {
            System.out.println("After fixing Phis------------------------");
            state.controlFlowGraph().print(System.out);
            System.out.println("End print--------------------------------");
        }
    }

    private void printPostAllocationMessage(MethodState state) {
        if (DEBUG) {
            System.out.println("After removing empty blocks--------------");
            state.controlFlowGraph().print(System.out);
            System.out.println("End print--------------------------------");
        }
    }
}
