/*
 * Class: StackOpt   
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.trans;

import org.apache.bcel.generic.*;

import edu.purdue.cs.bloat.cfg.FlowGraph;
import edu.purdue.cs.bloat.codegen.CodeGenerator;
import edu.purdue.cs.bloat.editor.InstructionAdapter;
import edu.purdue.cs.bloat.optimize.MethodState;
import edu.purdue.cs.bloat.optimize.Optimization;
import edu.purdue.cs.bloat.tree.Node;

public class StackOpt extends InstructionAdapter implements Optimization {

    int stackHeight;

    int minStackHeight;

    static final boolean DEBUG = false;

    public void transform(MethodState state) {
        // generate code without doing liveness or register allocation
        CodeGenerator codegen = new CodeGenerator(state);
        codegen.replacePhis().generateCode();

        // do stack optimization on the bytecode
        state.commitChanges();

        transform(state.methodGen());

        state.rebuildFlowGraph();
    }

    // StackOpt needs use and definition and label information to perform
    // transformations.
    // Consequently CodeGenerator must have been run to produce the code
    // supplied to StackOpt.transform.
    // Because of this I assume that labels and use and definition
    // information are in place.
    // This is currently implemented using attributes in InstructionHandles,
    // the keys being "Label" and
    // "Def". --Arrin
    private void transform(MethodGen method) {
        for (InstructionHandle i = method.getInstructionList().getEnd(); i != null; // && i
        // !=
        // method.getInstructionList().getStart();
        i = i.getPrev()) {

            Instruction inst = i.getInstruction();
            boolean isWide;

            if (inst instanceof LoadInstruction) {
                isWide = (((LoadInstruction) inst).getType(
                        method.getConstantPool()).getSize() == 2);
            } else if (isArrayLoadInstruction(inst)) {
                isWide = (((ArrayInstruction) inst).getType(
                        method.getConstantPool()).getSize() == 2);
            } else {
                continue;
            }

            if (DEBUG)
                System.out.println("Considering: " + i);

            stackHeight = 0;

            boolean seenLabel = FlowGraph.label(i); // If 'i' is a label we
            // don't do anything.
            for (InstructionHandle j = i.getPrev();; j = j.getPrev()) {

                // stop at the begining of the code or a basic block.
                // As Labels are also instructions
                if (j == null || seenLabel)
                    break;

                if (stackHeight == -1
                        && ((hasSameDef(i, j) && isLoadInstruction(j)) || dupRun(
                                j, i))) {
                    if (forwardCountCheck(method, j, i, -1)) {
                        // found a type 0 relation with a load
                        if (DEBUG) {
                            System.out.println("load type 0: " + j + " " + i);
                        }
                        if (isWide) {
                            method.getInstructionList().append(j, new DUP2());
                        } else {
                            method.getInstructionList().append(j, new DUP());
                            // add dup
                        }
                        i.setInstruction(new NOP());
                        // TODO: Why is deleting this a problem?
                        // I would have deleted it but it seems to cause all
                        // sorts of evil
                    }
                    break; // done, even if final check failed.
                } else if (stackHeight == 0 && hasSameDef(i, j)) {
                    if (isStoreInstruction(j)) {
                        if (forwardCountCheck(method, j, i, 0)) {
                            // found a type 0 with a store
                            if (DEBUG) {
                                System.out.println("store type 0: " + j + " "
                                        + i);
                            }
                            if (isWide) {
                                method.getInstructionList().append(j,
                                        j.getInstruction());
                                j.setInstruction(new DUP2());
                            } else {
                                method.getInstructionList().append(j,
                                        j.getInstruction());
                                j.setInstruction(new DUP());

                            }
                            i.setInstruction(new NOP());
                            // I would have deleted it but it seems to cause
                            // all sorts of evil

                        }
                        break;
                    } else if (isLoadInstruction(j) && !isWide) {
                        // can't do type 1s with wides.
                        if (forwardCountCheck(method, j, i, -1)) {
                            // found a type 1 with a load
                            if (DEBUG)
                                System.out.println("load type 1: " + j + " "
                                        + i);
                            method.getInstructionList().append(j, new DUP());
                            System.out.println("HERE1 inserting SWAP");
                            i.setInstruction(new SWAP());
                        }
                        break;
                    }
                } else if (stackHeight == 1 && hasSameDef(i, j)) {
                    if (isStoreInstruction(j) && !isWide) { // can't do type 1
                                                            // with wides

                        if (forwardCountCheck(method, j, i, 0)) {
                            // type 1 for stores
                            if (DEBUG)
                                System.out.println("store type 1: " + j + " "
                                        + i);
                            method.getInstructionList().append(j,
                                    j.getInstruction());
                            j.setInstruction(new DUP());
                            System.out.println("HERE2 inserting SWAP");
                            i.setInstruction(new SWAP());
                        }
                        break;
                    }
                }
                heightChange(method, j.getInstruction());

                if (FlowGraph.label(j)) {
                    seenLabel = true;
                }
            }
        }

        method.removeNOPs();
        method.getInstructionList().setPositions();
        if (DEBUG) {
            System.out.println("Method:");
            System.out.println(method);
            for (InstructionHandle ih = method.getInstructionList().getStart(); ih != null; ih = ih
                    .getNext()) {
                if (FlowGraph.label(ih))
                    System.out.print("(L)");
                else
                    System.out.print("   ");
                System.out.print(" " + ih.getPosition() + ": "
                        + ih.getInstruction());
                if (ih.getAttribute("Def") != null)
                    System.out.print("(" + ih.getAttribute("Def") + ")");
                System.out.println("");
            }
            // System.out.println(method.getMethod().getCode());
            System.out.println();
            System.out.println("LocalVariables: ");
            LocalVariableGen[] locals = method.getLocalVariables();
            for (int i = 0; i < locals.length; i++) {
                System.out.println("  " + locals[i]);
            }
            System.out.println("LocalVariable size: " + method.getMaxLocals());
            CodeExceptionGen[] excptns = method.getExceptionHandlers();
            if (excptns != null) {
                System.out.println("ExceptionHandlers: ");
                for (int j = 0; j < excptns.length; j++)
                    System.out.println("  " + excptns[j]);
            }
        }

        method.setMaxStack();
        method.setMaxLocals();
        method.update();
    }

    boolean forwardCountCheck(MethodGen m, InstructionHandle j,
            InstructionHandle i, int bound) {

        stackHeight = 0;
        minStackHeight = 0;

        for (InstructionHandle k = j.getNext(); k != i; k = k.getNext()) {
            heightChange(m, k.getInstruction());
            if (minStackHeight < bound)
                return false;
        }

        return true;
    }

    boolean dupRun(InstructionHandle j, InstructionHandle i) {
        InstructionHandle k = j;
        Instruction inst = k.getInstruction();

        if (!(inst instanceof DUP))
            return false;

        do {
            k = k.getPrev();
            inst = k.getInstruction();

            if (isLoadInstruction(inst) && hasSameDef(i, k)) {
                return true;
            }
        } while (!FlowGraph.label(k) && k.getInstruction() instanceof DUP);

        return (isLoadInstruction(inst)) && hasSameDef(i, k);
    }

    void heightChange(MethodGen method, Instruction inst) {
        stackHeight = stackHeight - inst.consumeStack(method.getConstantPool());

        if (stackHeight < minStackHeight)
            minStackHeight = stackHeight;

        stackHeight = stackHeight + inst.produceStack(method.getConstantPool());
    }

    private boolean hasSameDef(InstructionHandle i, InstructionHandle j) {
        Node iDef = (Node) i.getAttribute("Def");
        Node jDef = (Node) j.getAttribute("Def");
        return ((iDef != null) && (jDef != null) && iDef == jDef);
    }

    public String traceMessage(String dateString) {
        return "  New stack optimization: " + dateString;
    }

    public String preDebugMessage() {
        return null;
    }

    public String postDebugMessage() {
        return null;
    }

    private static boolean isLoadInstruction(InstructionHandle handle) {
        return isLoadInstruction(handle.getInstruction());
    }

    private static boolean isLoadInstruction(Instruction inst) {
        return inst instanceof LoadInstruction || isArrayLoadInstruction(inst);
    }

    private static boolean isArrayLoadInstruction(Instruction inst) {
        return inst instanceof IALOAD || inst instanceof FALOAD
                || inst instanceof AALOAD || inst instanceof BALOAD
                || inst instanceof CALOAD || inst instanceof LALOAD
                || inst instanceof DALOAD || inst instanceof SALOAD;
    }

    private static boolean isStoreInstruction(InstructionHandle handle) {
        return isStoreInstruction(handle.getInstruction());
    }

    private static boolean isStoreInstruction(Instruction inst) {
        return inst instanceof StoreInstruction
                || isArrayStoreInstruction(inst);
    }

    private static boolean isArrayStoreInstruction(Instruction inst) {
        return inst instanceof IASTORE || inst instanceof FASTORE
                || inst instanceof AASTORE || inst instanceof BASTORE
                || inst instanceof CASTORE || inst instanceof LASTORE
                || inst instanceof DASTORE || inst instanceof SASTORE;
    }

}
