/*
 * Class: ArrayLengthExpr   
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.tree;

import org.apache.bcel.generic.Type;

import edu.purdue.cs.bloat.util.Assert;

/**
 * <tt>ArrayLengthExpr</tt> represents the <i>arraylength</i> opcode which
 * gets length of an array.
 */
public class ArrayLengthExpr extends Expr {
    Expr array;

    /**
     * Constructor.
     * 
     * @param array
     *            Array whose length is sought.
     * @param type
     *            The type of this expression.
     */
    public ArrayLengthExpr(Expr array, Type type) {
        super(type);
        this.array = array;
        array.setParent(this);
    }

    public Expr array() {
        return array;
    }

    public void visitForceChildren(TreeVisitor visitor) {
        if (visitor.reverse()) {
            array.visit(visitor);
        } else {
            array.visit(visitor);
        }
    }

    public void visit(TreeVisitor visitor) {
        visitor.visitArrayLengthExpr(this);
    }

    public int exprHashCode() {
        return 3 + array.exprHashCode() ^ (int) Assert.simple(type).hashCode();
    }

    public boolean equalsExpr(Expr other) {
        return other != null && other instanceof ArrayLengthExpr
                && ((ArrayLengthExpr) other).array.equalsExpr(array);
    }

    public Object clone() {
        return copyInto(new ArrayLengthExpr((Expr) array.clone(), type));
    }
}
