/*
 * Class: LabelStmt    
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.tree;

import org.apache.bcel.generic.InstructionHandle;

import edu.purdue.cs.bloat.cfg.FlowGraph;

/**
 * LabelStmt is a placeholder in a Tree for a Label (the target of a jump).
 * 
 * BLOAT has a Label instruction the purpose of which is to denote the target of
 * a jump or BranchInstruction, BCEL has no such instruction. Many of the
 * optimizations in BLOAT and FlowGraph depend on theLabels being present. To
 * mimick the presence of Labels the BCEL port uses the method
 * buildBLOATInstructionList(MethodGen) in FlowGraph, this sets
 * InstructionHandle attributes using the key "Label". The methods setLabel and
 * unsetLabel are supplied to manipulate this state and the method label returns
 * a boolean true if the instruction is also a label. This has one complication
 * not present in BLOAT, that is an instruction in the port can be a "Label"
 * (FlowGraph.label() returns true) and also have an instruction which serves a
 * useful purpose(is not a NOP instruction). When the FlowGraph is commited in
 * CodeGenerator, the BCEL port uses NOP instructions to commit LabelStmts. For
 * robustness later optimizations performed on bytecode (Peephole and StackOpt)
 * do not rely on labels also being NOP instructions. --Arrin
 * 
 * @see InstructionHandle
 * @see FlowGraph#label
 * @see FlowGraph#setLabel
 * @see FlowGraph#unsetLabel
 * @see FlowGraph#buildBLOATInstructionList
 * @see Tree#addLabel
 */
public class LabelStmt extends Stmt {
    final InstructionHandle label;

    /**
     * Constructor.
     * 
     * @param label
     *            The label(InstructionHandle) that comprises this statement.
     */
    public LabelStmt(InstructionHandle label) {
        this.label = label;
    }

    public InstructionHandle label() {
        return label;
    }

    public void visitForceChildren(TreeVisitor visitor) {
    }

    public void visit(TreeVisitor visitor) {
        visitor.visitLabelStmt(this);
    }

    public Object clone() {
        return copyInto(new LabelStmt(label));
    }
}
