/*
 * Class: MonitorStmt    
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.tree;

/**
 * MonitorStmt represents the <tt>monitorenter</tt> and <tt>monitorexit</tt>
 * opcodes, which gain and release ownership of the monitor associated with a
 * given object.
 */
public class MonitorStmt extends Stmt {
    public static final int ENTER = 0;

    public static final int EXIT = 1;

    final int kind;

    Expr object;

    /**
     * Constructor.
     * 
     * @param kind
     *            The kind of monitor statement: ENTER or EXIT.
     * @param object
     *            The expression (object) whose monitor is being entered or
     *            exited.
     */
    public MonitorStmt(int kind, Expr object) {
        this.kind = kind;
        this.object = object;

        object.setParent(this);
    }

    public Expr object() {
        return object;
    }

    public int kind() {
        return kind;
    }

    public void visitForceChildren(TreeVisitor visitor) {
        object.visit(visitor);
    }

    public void visit(TreeVisitor visitor) {
        visitor.visitMonitorStmt(this);
    }

    public Object clone() {
        return copyInto(new MonitorStmt(kind, (Expr) object.clone()));
    }
}
