/*
 * Class: Assert    
 * Version: 
 * Date:
 * 
 * All files in the distribution of BLOAT (Bytecode Level Optimization and
 * Analysis tool for Java(tm)) are Copyright 1997-2001 by the Purdue
 * Research Foundation of Purdue University.  All rights reserved.
 *
 * <p>
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this entire copyright notice is duplicated in all
 * such copies, and that any documentation, announcements, and other
 * materials related to such distribution and use acknowledge that the
 * software was developed at Purdue University, West Lafayette, IN by
 * @author Antony Hosking
 * @author David Whitlock
 * @author Nathaniel Nystrom
 * No charge may be made for copies, derivations, or distributions of
 * this material without the express written consent of the copyright
 * holder.  Neither the name of the University nor the name of the
 * author may be used to endorse or promote products derived from this
 * material without specific prior written permission.  THIS SOFTWARE
 * IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 * <p>
 * 
 * Port of the BLOAT to use the Jakata Apache project BCEL 5.2 was 
 * contributed by The Australian National University by
 * @author Arrin Daley
 * @author John N Zigman
 * 
 * <p>
 * Java is a trademark of Sun Microsystems, Inc.
 */

package edu.purdue.cs.bloat.util;

import org.apache.bcel.generic.*;
import org.apache.bcel.Constants;

/**
 * Mechanism for making assertions about things in BLOAT. If an assertion fails,
 * an <tt>IllegalArgumentException</tt> is thrown.
 */
public abstract class Assert {
    public static void isTrue(boolean test, String msg) {
        if (!test) {
            // try{
            throw new IllegalArgumentException("Assert.isTrue: " + msg);
            // }catch(IllegalArgumentException iae){
            // iae.printStackTrace();
            // throw iae;
            // }
        }
    }

    public static void isFalse(boolean test, String msg) {
        if (test) {
            throw new IllegalArgumentException("Assert.isFalse: " + msg);
        }
    }

    public static void isNotNull(Object test, String msg) {
        if (test == null) {
            throw new IllegalArgumentException("Assert.isNotNull: " + msg);
        }
    }

    public static void isNull(Object test, String msg) {
        if (test != null) {
            throw new IllegalArgumentException("Assert.isNull: " + msg);
        }
    }

    public static void isTrue(boolean test) {
        if (!test) {
            throw new IllegalArgumentException("Assert.isTrue failed");
        }
    }

    public static void isFalse(boolean test) {
        if (test) {
            throw new IllegalArgumentException("Assert.isFalse failed");
        }
    }

    public static void isNotNull(Object test) {
        if (test == null) {
            throw new IllegalArgumentException("Assert.isNotNull failed");
        }
    }

    public static void isNull(Object test) {
        if (test != null) {
            throw new IllegalArgumentException("Assert.isNull failed");
        }
    }

    /**
     * A method I couldn't find a BCEL version of if the type will fit into an
     * integer it returns Type.INT, if the type is a subclass of object (ie not
     * a basic type) it returns Type.OBJECT, otherwise it returns the type
     * given.
     */
    public static Type simple(Type type) {
        if (isIntegral(type))
            return Type.INT;
        if (type instanceof ReferenceType)
            return Type.OBJECT;
        return type;
    }

    /**
     * A method I couldn't find a BCEL version of, returns true if the type will
     * fit into an integer false otherwise.
     */
    public static boolean isIntegral(Type type) {
        byte t = type.getType();
        return ((t == Constants.T_INT) || (t == Constants.T_CHAR)
                || (t == Constants.T_SHORT) || (t == Constants.T_BOOLEAN) || (t == Constants.T_BYTE));
    }

}
